package com.iflytek.cloud;

import android.os.Bundle;
/**
 * <h3>数据请求监听器</h3>
 * <script>gShowExtraFace();</script>
 * <p>通过实现此接口，获取请求的状态和结果</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see FaceRequest
 */
public abstract interface RequestListener {
	/**
	 * <h4>事件回调</h4>
	 * <p>
	 * 		用于接收扩展事件消息的函数。请参考{@link SpeechEvent}类。
	 * </p>
	 * 
	 * @param eventType 消息类型，参考{@link SpeechEvent}类的事件ID。
	 * @param params 消息数据对象，对象内容因事件而异，参考
	 * 		{@link SpeechEvent}类的事件说明。
	 * 
	 * @see SpeechEvent
	 */
	void onEvent(int eventType,Bundle params);
	
	/**
	 * <h4>结果数据回调</h4>
	 * <p>
	 * 		此函数可能回调多次，但在{@link #onCompleted}回调错误时，将不回调此函数。
	 * </p> 
	 * @param buffer 服务端返回的二进制数据，数据内各键值，请参考<script>gShowUserGuideFile();</script>。
	 * @see #onCompleted
	 */
	void onBufferReceived(byte[] buffer);
	
	/**
	 * <h4>结束回调</h4>
	 * <p>
	 * 		回调此函数时，会话完成，若会话有错误，则通过参数1返回错误信息。
	 * </p>
	 * @param error 错误信息，会话正常时，error值为null。
	 * 
	 * @see SpeechError
	 */
	void onCompleted(SpeechError error);
}
