package com.iflytek.cloud;

import android.os.Bundle;



/**
 * <h3>唤醒回调接口</h3>
 * <p>
 * 		通过实现此接口，获取唤醒的状态和结果。
 * </p>
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see VoiceWakeuper
 */
public abstract interface WakeuperListener{
	/**
	 * <h4>开始说话</h4>
	 * <p>
	 * 		在录音模式({@link SpeechConstant#AUDIO_SOURCE 音频源}参数设为 > -1时 )下，
	 * 调用{@link SpeechRecognizer#startListening 开始录音函数}后，会自动开启系统的录音
	 * 机，并在录音机开启后，会回调此函数（这中间的过程应该在几毫秒内，可以忽略，除非系
	 * 统响应很慢）。
	 * </p>
	 * <p>
	 * 		应用层可通过此函数回调，告知用户，当前可开始说话。若应用层使用的是声音提示，
	 * 则应该在调用{@link VoiceWakeuper#startListening 开始录音函数}前，提放提示音，
	 * 并在提示音非静音数据播放结束时，调用
	 * {@link VoiceWakeuper#startListening 开始录音函数}，开始录音。若太早调用，则
	 * 可能会把提示音录进要识别的音频中，若太晚，则可能会漏掉部分用户说话的音频。
	 * </p>
	 * @see VoiceWakeuper#startListening
	 */
	void onBeginOfSpeech();

	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。当出现{@link #onError}错误，或取消会话时，
	 * 可能不会再回调此函数返回结果。
	 * </p>
	 * <p>
	 * 		单次唤醒：在{@link SpeechConstant#KEEP_ALIVE}参数设为非持续唤醒时，若录入的音频
	 * 与唤醒资源匹配，返回一次结果，并自动停止录音，并结束当次会话。
	 * </p>
	 * <p>
	 * 		持续唤醒：在{@link SpeechConstant#KEEP_ALIVE}参数设为持续唤醒时，若录入的音频
	 * 与唤醒资源匹配，返回结果，录音继续，会话也在继续，直到应用主动调用
	 * {@link VoiceWakeuper#stopListening}停止录音，或{@link VoiceWakeuper#cancel}取消
	 * 会话，或出现{@link #onError}错误，而导致会话结束为止。即，会多次返回结果——若
	 * 录音数据与唤醒资源匹配。
	 * </p>
	 * <p>
	 * 		唤醒识别：通过此函数返回一次唤醒结果，然后再通过{@link #onEvent}返回一次识别结果，
	 * 并自动停止录音，结束会话。参考{@link #onEvent}。
	 * </p>
	 * <p>
	 * 		特定人唤醒注册：根据设置的{@link SpeechConstant#IVW_ENROLL_TIMES 注册次数}，返回
	 * 与之相同次数的结果。
	 * </p>
	 * <p>
	 * 		识别采用边录边上传的分次上传音频数据方式，可能在结束录音前，就有结果返回。
	 * </p>
	 * 
	 * @param result 结果数据
	 * @see #onError
	 * @see VoiceWakeuper#cancel
	 * @see WakeuperResult
	 * @see VoiceWakeuper#writeAudio
	 * @see #onEvent
	 */
	void onResult(WakeuperResult result);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，录音也会停止。应在再次调用
	 * {@link VoiceWakeuper#startListening}开启新的会话前，
	 * 停止调用{@link VoiceWakeuper#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * </p>
	 * @param error 错误类型
	 * @see SpeechError
	 * @see #onResult
	 * @see VoiceWakeuper#startListening
	 */
	void onError(SpeechError error);
	
    /**
     * <h4>事件</h4>
     * <p>
     * 		扩展用接口，唤醒的主要事件是音频事件，以及在唤醒识别时，返回识别结果（在唤醒结果之后返回）。
     * </p>
     * <p>
     * 		识别结果事件ID（消息类型）为{@link SpeechEvent#EVENT_IVW_RESULT}，通过第二
     * 个参数，返回是否最后一个识别结果（0为否，1为是）。结果内容则从第四个参数
     * 中获取，通过{@link android.os.Bundle#get(String)}，指定参数为
     * {@link SpeechEvent#KEY_EVENT_IVW_RESULT},获取{@link RecognizerResult}对象。
     * </p>
     * <p>
     * 		音频事件ID（消息类型）为 {@link SpeechEvent#EVENT_RECORD_DATA}，
     * 若 {@link SpeechConstant#IVW_CHANNEL_NUM} 不为0，则在唤醒后返回
     * 从多声道的原始音频中，分解出来用于识别业务的音频（另参考{@link SpeechConstant#IVW_RESET_AIMIC}）；
     * 否则返回当前设备录音的音频（{@link SpeechConstant#NOTIFY_RECORD_DATA} 参数为真时）。
     * </p>
     * <p>
     * 		应用可以使用音频事件返回的音频，调用 {@link SpeechRecognizer} 进行识别等。
     * </p>
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 消息内容
     * 
     * @see SpeechEvent#EVENT_IVW_RESULT
     * @see SpeechEvent#KEY_EVENT_IVW_RESULT
     * @see SpeechEvent#EVENT_RECORD_DATA
     * @see SpeechConstant#IVW_RESET_AIMIC
     * @see RecognizerResult
     * @see #onResult
     */
    void onEvent(int eventType, int arg1,int arg2, Bundle obj);
    
	/**
	 * <h4>音量变化</h4>
	 * <p>
	 *     当开始录音，到停止录音（停止写入音频流）为止，
	 *  SDK检测到音频数据（正在录音或写入音频流）的音量变化时，会多次通过此函数回调，告
	 *  知应用层当前的音量值。
	 * </p> 
	 * @param volume 当前音量值，范围[0-30]
	 * 
	 * @see VoiceWakeuper#startListening
	 * @see #onBeginOfSpeech
	 * @see VoiceWakeuper#stopListening
	 */
	void onVolumeChanged(int volume);
}
