package com.iflytek.cloud.param;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.SystemClock;
import android.telephony.TelephonyManager;
import android.telephony.cdma.CdmaCellLocation;
import android.telephony.gsm.GsmCellLocation;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.Setting;
import com.iflytek.cloud.Setting.LOG_LEVEL;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.Version;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.util.AppInfoUtil;
import com.iflytek.cloud.msc.util.Config;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.UniqueIDUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class ParamBuilder{
	 
	// 服务端返回结果类型
	private static final String RST_JSON = "json";
	// speex-wb默认压缩等级
	private static final String AUE_SPEEX_WB = "speex-wb";
	// speex默认压缩等级
	private static final String AUE_SPEEX = "speex";
	// 未压缩
	private static final String AUE_RAW = "raw";
	
	// 转写
	// 听写
	private static final String SUB_IAT = MscKeys.SUB_IAT;
	// 识别
	private static final String SUB_ASR = MscKeys.SUB_ASR;
	// 声纹
	private static final String SUB_IVP = MscKeys.SUB_IVP;
	// 评测
	private static final String SUB_SEE = MscKeys.SUB_SEE;
	// 转写
	private static final String SUB_IST = MscKeys.SUB_IST;
	
	// 身份验证
	private static final String SUB_MFV = MscKeys.SUB_MFV;
	
	private static final String VALUE_TRUE = "1";
	
	private static final String VALUE_FALSE = "0";
	
	// DOMAIN 值
	public static final  String DOMAIN_SMS = "sms";
	public static final  String DOMAIN_IAT = "iat";
	
	public static final String DEF_LANG = "zh_cn";
	public static final String DEF_ACCENT = "mandarin";
	
	// 默认日志保存路径
	public static final  String LOG_PATH = "/sdcard/msc/msc.log";
	
	public static final int DEF_BOS_LONG = 5000;
	public static final int DEF_BOS_SHORT= 4000;
	public static final int DEF_EOS_LONG = 1800;
	public static final int DEF_EOS_SHORT = 700;
	
	//合成默认发音人设置
	private static String   DEFAULT_VOICE_NAME = "xiaoyan";
	
	private static String sNetType = null;
	
	/**
	 * 
	 * @param context 设备上下文
	 * @param looper	参数
	 * @return String string类型参数组合
	 * @throws SpeechError 错误
	 */
	public static String composeInitParam(Context context,MscHandler looper) throws SpeechError {
		
//		// 检查appid传入值是否和动态库相符合
//		if(param.getString(MscKeys.APPID) != null)
//			throw new SpeechError(SpeechError.ERROR_INVALID_PARAM,SpeechError.UNKNOWN);

		// 检查上下文句柄
		if(context == null)
			throw new SpeechError(ErrorCode.ERROR_INVALID_PARAM);
		
		HashParam param = looper.getParam().clone();
		
		sNetType = param.getString( SpeechConstant.NET_TYPE, sNetType );
		
		// 网络类型参数
		appendNetProxyParam(context,param);
		
		// 默认超时时间20s
		param.putParam(SpeechConstant.NET_TIMEOUT,"20000",false);

		param.putParam(MscKeys.MSP_AUTH,VALUE_TRUE,false);

		// 控件版本号
		param.putParam(MscKeys.MSC_VER,Version.getVersion());
		
		// MAC地址
		HashParam apparam = AppInfoUtil.getAppInfo(context);
		param.putParam(MscKeys.MAC_ADDR, apparam.getString(AppInfoUtil.NET_MAC),false);

		// 设备信息，不做覆盖
		param.putParam(MscKeys.DVC,getDeviceId(context),false);
		
		// Unique ID
		param.putParam(MscKeys.UNIQUE_ID, UniqueIDUtil.getUniqueID(context));
		
		// 插入经纬度信息
		//param.putParam(Config.KEY_LATITUDE,""+Config.getConfig(context).getLocation(Config.KEY_LATITUDE),false);
		//param.putParam(Config.KEY_LONGITUDE,""+Config.getConfig(context).getLocation(Config.KEY_LONGITUDE),false);

		// 插入设备信息
		param.putHash(AppInfoUtil.getShortAppInfo(context));   
		
		// 插入日志参数 
		appendLogParam(param);
		
		// 插入基站信息
		appendMMLCParam(context,param);
		
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();
	}
	
	/**
	 * 增加日志参数
	 * @param param
	 */
	private static void appendLogParam(HashParam param)
	{
		// none级别不保存日志
		if(param == null || Setting.getLogLevel() == LOG_LEVEL.none)
			return;
		String path = Setting.getLogPath();
		if(TextUtils.isEmpty(path))
			path = LOG_PATH;
		int level = -1;
		if(Setting.getLogLevel() == LOG_LEVEL.detail)
			level = 31;
		else if(Setting.getLogLevel() == LOG_LEVEL.normal)
			level = 15;
		else if(Setting.getLogLevel() == LOG_LEVEL.low)
			level = 7;
		// 预先创建路径
		FileUtil.makeDir(path);
		param.putParam(MscKeys.KEY_LOG_PATH, path);
		param.putParam(MscKeys.KEY_LOG_LEVEL,""+level);
		// 日志只输出到log文件中
		// 0->不打印日志，1->输出到文件，2->控制台，3->文件，控制台都打印
		param.putParam(MscKeys.KEY_LOG_OUTPUT, VALUE_TRUE, false);
	}

	/**
	 * 增加网络类型参数wap_proxy,net_subtype
	 * @param context
	 * @param param
	 */
	public static void appendNetProxyParam(Context context,HashParam param)
	{
		final String netType = param.getString( SpeechConstant.NET_TYPE );
		if( TextUtils.isEmpty(netType) && !TextUtils.isEmpty(sNetType) ){
			param.putParam(SpeechConstant.NET_TYPE, sNetType, false);
		}else{
			if(context == null)
			{
				param.putParam(SpeechConstant.NET_TYPE, NetworkUtil.NET_UNKNOWN,false);
			}else
			{
				try{
					ConnectivityManager  nmgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
					NetworkInfo active = nmgr.getActiveNetworkInfo();
					if(active == null)
					{
						param.putParam(SpeechConstant.NET_TYPE, NetworkUtil.NET_UNKNOWN,false);
					}else
					{
						param.putParam(SpeechConstant.NET_TYPE,  NetworkUtil.getNetType(active),false);
						String subnet = NetworkUtil.getNetSubType(active);
						param.putParam(MscKeys.NET_SUBTYPE,HashParam.trimInvalid(subnet),false);
					}
				}catch( SecurityException e ){
					DebugLog.LogI( "appendNetProxyParam exceptoin: "+e.getLocalizedMessage() );
				}catch( Throwable e ){
					DebugLog.LogE( e );
				}//end of try-catch
			}
		}//end of if-else sNetType not empty
		

	}
	
	/**
	 * 获取基站相关信息：mmlc = MCC|MNC|LAC|CID
	 *  MCC，Mobile Country Code，移动国家代码（中国的为460）；
	 *	MNC，Mobile Network Code，移动网络号码（中国移动为0，中国联通为1，中国电信为3）； 
	 *  LAC，Location Area Code，位置区域码；
	 *	CID，Cell Identity，基站编号；
	 * @param context
	 * @param param
	 */
	public static void appendMMLCParam(Context context,HashParam param){
		long d1 = SystemClock.elapsedRealtime();
		if(context == null)
			return;

		if(!Config.hasGPSPermission(context))
			return;
		
		try {

			TelephonyManager mTelephonyManager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);  
			int phoneType = mTelephonyManager.getPhoneType();
			// 返回值MCC + MNC  
			String operator = mTelephonyManager.getNetworkOperator();  
			int mcc = Integer.parseInt(operator.substring(0, 3));  
			int mnc = Integer.parseInt(operator.substring(3));  

			int cellId = 0 ;  
			int lac = 0;
			switch (phoneType) {
			case TelephonyManager.PHONE_TYPE_CDMA:
				CdmaCellLocation location1 = (CdmaCellLocation) mTelephonyManager.getCellLocation(); 
				cellId = location1.getBaseStationId(); 
				lac = location1.getNetworkId(); 
				break;
			case TelephonyManager.PHONE_TYPE_GSM:
				GsmCellLocation location = (GsmCellLocation) mTelephonyManager.getCellLocation();  
				cellId = location.getCid();
				lac = location.getLac();
			default:

				break;
			}
			param.putParam(MscKeys.MMLC, mcc+"|"+mnc+"|"+lac+"|"+cellId);
			DebugLog.LogS("MCC = " + mcc + "\t MNC = " + mnc + "\t phoneType = " + phoneType + "\t LAC = "+ lac + "\t CID = " + cellId );  
		} catch (Exception e) {
			DebugLog.LogS ("get mmlc failed");  
		}
		DebugLog.LogS("get mmlc time cost:" + (SystemClock.elapsedRealtime() - d1));
	}

	/**
	 * 增加活动页面信息
	 * @param context
	 * @param param
	 */
	public static void addActivityParam(Context context,HashParam param) {
		String class_name = AppInfoUtil.getActivityMsg(context);
		param.putParam(MscKeys.ACTIVITY_NAME,class_name);
	}

	/**
	 * 初始化转写参数
	 * @param context 上下文
	 * @param grammar 语法
	 * @param looper handler对象
	 * @return
	 */
	public static String composeIsrSessionParam(Context context,String grammar,MscHandler looper) {
		HashParam param = looper.getParam().clone();
		//此字段可能是XML等自定义格式，可能含有逗号等
		param.removeParam(SpeechConstant.CLOUD_GRAMMAR);
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		// 活动页面信息
		//addActivityParam(context,param);
		
		//没有设置语言时默认添加，否则服务端报错10303 
		param.putParam(SpeechConstant.LANGUAGE, DEF_LANG, false);
		param.putParam( SpeechConstant.ACCENT, DEF_ACCENT, false );
		
		param.putParam(SpeechConstant.RESULT_TYPE, RST_JSON,false);
		param.putParam(MscKeys.KEY_RSE, looper.getResultEncoding(),false);
		param.putParam(SpeechConstant.TEXT_ENCODING, looper.getTextEncoding(),false);
		
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE,false);
		//默认未使用UI
		param.putParam(MscKeys.MSC_SKIN, VALUE_FALSE,false);
		
		// set sub as iat or asr
		if (TextUtils.isEmpty(grammar)) 
		{
			param.putParam(SpeechConstant.SUBJECT, SUB_IAT,false);
		}else
			param.putParam(SpeechConstant.SUBJECT, SUB_ASR,false);
		
		// set Record rate
		int rate = looper.getSampleRate();
		param.putParam(MscKeys.AUDIO_AUF, Integer.toString(rate),false);
		
		if(rate == PcmRecorder.RATE16K)
		{
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX_WB, false);
		}else {
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX, false);
		}
		
		param.putParam(SpeechConstant.VAD_BOS, Integer.toString(getDefaultBos(looper)), false);
		param.putParam(SpeechConstant.VAD_EOS, Integer.toString(getDefaultEos(looper)), false);
		
		if( param.getBoolean(SpeechConstant.DVC_INFO, false) ){
			// 网络类型参数
			appendNetProxyParam(context,param);
			
			// 设备信息，不做覆盖
			param.putParam(MscKeys.DVC,getDeviceId(context),false);
			
			// Unique ID
			param.putParam(MscKeys.UNIQUE_ID, UniqueIDUtil.getUniqueID(context));
			
			// 插入经纬度信息
			param.putParam(Config.KEY_LATITUDE,""+Config.getConfig(context).getLocation(Config.KEY_LATITUDE),false);
			param.putParam(Config.KEY_LONGITUDE,""+Config.getConfig(context).getLocation(Config.KEY_LONGITUDE),false);
			
			final String usrAgent = AppInfoUtil.checkAndReplaceIllegalSymbols( AppInfoUtil.getBrowserUserAgent(context) );
			final String dvcType = AppInfoUtil.getDeviceType(context);
			param.putParam( AppInfoUtil.BROWSER_UA, usrAgent, false );
			param.putParam( AppInfoUtil.DVC_TYPE, dvcType, false );

			// 插入设备信息
			param.putHash(AppInfoUtil.getShortAppInfo(context));   
		}
		
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();
	}
	/**
	 * 初始化唤醒参数
	 * @param context 上下文
	 * @param sessionType 唤醒类型，持续唤醒还是一次唤醒
	 * @param looper handler对象
	 * @return
	 */
	public static String composeIvwSessionParam(Context context,String sessionType,MscHandler looper) {
		if (sessionType.equals(MscKeys.IVW_ONESHOT)) 
		{
			//唤醒+识别
			return composeIsrSessionParam(context, looper.getParam().getString(SpeechConstant.CLOUD_GRAMMAR), looper);
		}
		else 
		{
			HashParam param = looper.getParam().clone();
			
			int net_mode = param.getInt(SpeechConstant.IVW_NET_MODE, MscKeys.IVW_MODE_CLOSE);
			if(MscKeys.IVW_MODE_CLOSE == net_mode ){
				param.putParam(SpeechConstant.IVW_NET_MODE, VALUE_FALSE, true);
			} else if(MscKeys.IVW_NORMAL_QUERY == net_mode 
					|| MscKeys.IVW_NORMAL_UPDATA == net_mode){ 
				param.putParam(SpeechConstant.IVW_NET_MODE, VALUE_TRUE, true);
			} else if (NetworkUtil.isWifiConnect(context)){
				param.putParam(SpeechConstant.IVW_NET_MODE, VALUE_TRUE, true);
			} else {
				param.putParam(SpeechConstant.IVW_NET_MODE, VALUE_FALSE, true);
			}
			// 映射为内部参数
			param.replaceKey(MscKeys.MSC_MAP_KEYS);
			return param.toString();	
		}
	}
	/**
	 * 初始化转写参数
	 * @param context 上下文
	 * @param extra 额外参数
	 * @param looper handle对象
	 * @return
	 */
	public static String composeIstSessionParam(Context context,String extra,MscHandler looper) {
		HashParam param = looper.getParam();
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		
		param.putParam( SpeechConstant.RESULT_TYPE, RST_JSON,false );
		param.putParam( MscKeys.KEY_RSE, DataUtil.GB2312,false ); //转写结果编码为GB2312
		param.putParam( SpeechConstant.TEXT_ENCODING, looper.getTextEncoding(),false );
		param.putParam( "vad_enable", "false", false );
		
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE,false);

		// set sub as iat or asr
		param.putParam( "sub", SUB_IST, false );
		 
		// set Record rate
		int rate = looper.getSampleRate();
		param.putParam(MscKeys.AUDIO_AUF, Integer.toString(rate),false);
		
		if(rate == PcmRecorder.RATE16K)
		{
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX_WB, false);
		}else {
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX, false);
		}
		
		return param.toString();
	}
	
	/**
	 * 初始化声纹参数
	 * @param context 上下文
	 * @param looper handle对象
	 * @return
	 */
	public static String composeIsvSessionParam(Context context,MscHandler looper) {
		HashParam param = looper.getParam().clone();
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		
		param.putParam(SpeechConstant.RESULT_TYPE, RST_JSON);
		param.putParam(MscKeys.KEY_RSE, looper.getResultEncoding());
		param.putParam(SpeechConstant.TEXT_ENCODING, looper.getTextEncoding());
		
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE,false);
		// set sub as ivt
		param.putParam(SpeechConstant.SUBJECT, SUB_IVP,false);
		
		// set Record rate
		int rate = looper.getSampleRate();
		param.putParam(MscKeys.AUDIO_AUF, Integer.toString(rate),false);
		
		if(rate == PcmRecorder.RATE16K)
		{
			//声纹默认为10级别，保证效果
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX_WB+";10", false);
		}else {
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX, false);
		}
		
		// 设置默认VAD时间
		param.putParam(SpeechConstant.VAD_BOS, "3000",false);
		param.putParam(SpeechConstant.VAD_EOS, "700",false);
	
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();
	}
	
	/**
	 * 初始化人脸识别参数
	 * @param context 上下文
	 * @param hashparam HashParam类型参数
	 * @return
	 */
	public static String composeWfrSessionParam(Context context,HashParam hashparam) {
		HashParam param = hashparam.clone();
		// 活动页面信息
		//addActivityParam(context,param);
		//put appid
		final SpeechUtility utility = SpeechUtility.getUtility();
		if( null != utility ){
			String appid = utility.getParameter(SpeechConstant.APPID);
			param.putParam(SpeechConstant.APPID, appid);
		}
		
		//put 设备信息
		param.putHash(AppInfoUtil.getShortAppInfo(context));
		param.putParam(MscKeys.DVC,ParamBuilder.getDeviceId(context),false);
		//put aue
		param.putParam(MscKeys.AUDIO_AUE, AUE_RAW, false);
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();	
	}
	
	/**
	 * 初始化唤醒闭环优化设备信息
	 * @param context 上下文
	 * @param hashparam HashParam类型参数
	 * @return
	 */
	public static String composeIvwResSearchParam(Context context,HashParam hashparam) {
		HashParam param = hashparam.clone();
		//put appid
		String appid = SpeechUtility.getUtility().getParameter(SpeechConstant.APPID);
		param.putParam(SpeechConstant.APPID, appid);
		//put 设备信息
		param.putHash(AppInfoUtil.getShortAppInfo(context));
		param.putParam(MscKeys.DVC,ParamBuilder.getDeviceId(context),false);
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();	
	}
	
	/**
	 * 初始化身份验证参数
	 * @param context 上下文
	 * @param grammar 语法
	 * @param looper handle对象
	 * @return
	 */
	public static String composeMfvSessionParam(Context context, String grammar, MscHandler looper) {
		HashParam param = looper.getParam().clone();
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		// 使用mfv通道
		param.putParam("sub", SUB_MFV, false);
		param.putParam("prot_ver", "50", false);
		// 协议版本设置为2.0，返回精简后的结果
		param.putParam("mver", "2.0", false);
		// MFV独立url
		//param.putParam(MscKeys.SERVER_URL,"http://imfv.openspeech.cn/msp.do", false);
		
		// 通过sst来设置场景，从而不需要开发者直接设置
		final String sst = param.getString("sst");
		if ("verify".equals(sst)) {
			param.putParam("scene_mode", "vfy", false);
		} else {
			param.putParam("scene_mode", "gen", false);
		}
		
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();
	}
	/**
	 * 初始化MSP接口参数
	 * @param context 上下文
	 * @param looper handle对象
	 * @return
	 */
	public static String composeMspSessionParam(Context context,MscHandler looper) {
		HashParam param = looper.getParam().clone();
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		// 活动页面信息
		//addActivityParam(context,param);
		
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE,false);

		param.putParam(SpeechConstant.RESULT_TYPE, RST_JSON,false);

		param.putParam(MscKeys.KEY_RSE, looper.getResultEncoding(),false);

		// 上传数据编码
		param.putParam(SpeechConstant.TEXT_ENCODING, looper.getTextEncoding(),false);
		
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		
		return param.toString();
	}

	/**
	 * 初始化合成参数
	 * @param context 上下文
	 * @param looper handle对象
	 * @return
	 */
	public static String composeTtsSessionParam(Context	context,MscHandler looper) {
		HashParam param = looper.getParam().clone();
		// 网络类型参数
		appendNetProxyParam(context,param);
		// 基站信息参数
		appendMMLCParam(context,param);
		// 活动页面信息
		//addActivityParam(context,param);
		
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE,false);
		

		// 采样率
		int rate = looper.getSampleRate();
		param.putParam(MscKeys.AUDIO_AUF, Integer.toString(rate));
		
		if(rate == PcmRecorder.RATE16K)
		{
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX_WB, false);
		}else {
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX, false);
		}
		
		//发音人设置，如果开发者未设置使用默认值
		String vcn =param.getString(SpeechConstant.VOICE_NAME, DEFAULT_VOICE_NAME);
		param.putParam(SpeechConstant.VOICE_NAME, vcn, true);

		// 文本编码
		param.putParam(SpeechConstant.TEXT_ENCODING, looper.getTextEncoding(),false);
		
		// 语速扩展
		speedIncreaseAdaptive(param);
		
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		return param.toString();
	}
	
	/**DTS 2013-06-18 add initialize ise params **************begin*/
	/**
	 * Initialize ISE params
	 * 
	 * @param context 上下文
	 * @param looper handle对象
	 * @return
	 */
	public static String composeIseSessionParam(Context context,MscHandler looper) {
		HashParam param = looper.getParam().clone();
		//set network type, general parameter
		appendNetProxyParam(context, param);
		// 基站信息参数
		appendMMLCParam(context,param);
		// 活动页面信息
		//addActivityParam(context,param);
		//set conversation mode, general parameter
		param.putParam(MscKeys.MSP_SSM, VALUE_TRUE, false);
		
		//set record rate
		int rate = looper.getSampleRate();
		param.putParam(MscKeys.AUDIO_AUF, Integer.toString(rate), false);
		
		if(rate == PcmRecorder.RATE16K)
		{
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX_WB, false);
		}else {
			param.putParam(MscKeys.AUDIO_AUE, AUE_SPEEX, false);
		}
		
//		//TODO : set answer type, should we need to provided default value if user do not set SST
//		param.putParam(MscKeys.KEY_ISE_SST, SpeechConfig.paramIseSst(), false);
//		
//		//TODO ； set review question type, shoule we need to provided default value? if user do not set cat
//		param.putParam(MscKeys.KEY_ISE_CATEGORY, SpeechConfig.paramIseCategory(), false);

		//set text encoding
		param.putParam(SpeechConstant.TEXT_ENCODING, looper.getTextEncoding(), false);
		// 返回精简结果
		param.putParam("plev", "1", false);
		param.putParam(SpeechConstant.ACCENT, "mandarin", false);
		param.putParam(SpeechConstant.DOMAIN, SUB_SEE, false);
		//set request type
		param.putParam(SpeechConstant.SUBJECT, SUB_SEE, false);
		param.putParam(SpeechConstant.RESULT_TYPE, "xml", false);
		
		// not clear seted VAD time
		param.putParam(SpeechConstant.VAD_BOS, "5000",false);
		param.putParam(SpeechConstant.VAD_EOS, "1800",false);

		param.putParam("vad_speech_timeout", Integer.MAX_VALUE + "", false);
		// 映射为内部参数
		param.replaceKey(MscKeys.MSC_MAP_KEYS);
		
		return param.toString();
	}
	/**DTS 2013-06-18 add initialize ise params **************end*/
	
	
	
	/**
	 * 获得网络类型参数
	 * @param svcmode
	 * @return
	 */
	public static String svcParam(int svcmode)
	{
		String ret = "";
		if(svcmode == 0)		// net only
			ret = "svc_mode=0,net_enabled=true";
		else if(svcmode == 1)	// net first
			ret = "svc_mode=1,net_enabled=true";
		else if(svcmode == 2)	// local only
			ret = "svc_mode=2,net_enabled=false";
		return ret;
	}
	
	/**
	 * 获取设备ID
	 * @param context
	 * @return
	 */
	public static String getDeviceId(Context context) {
		
		if(context == null)
			return "null";
		
		HashParam apparam = AppInfoUtil.getAppInfo(context);
		// 检查dvc参数获取是否正常
		String dvc = apparam.getString(AppInfoUtil.OS_IMSI) + "|" +
					 apparam.getString(AppInfoUtil.OS_IMEI);
		// dvc长度小于10，返回Mac地址
		if(dvc.length() < 10)
			dvc = apparam.getString(AppInfoUtil.NET_MAC);
		
		if(!TextUtils.isEmpty(dvc) && dvc.length() > 0)
			return dvc;
		else
			return null;
	}
	
	/**
	 * 获得打开url链接所携带的参数
	 * @return
	 */
	public static String getUrlParam(Context context)
	{
		String param = "";
		HashParam apparam = AppInfoUtil.getAppInfo(context);
		String imei = apparam.getString(AppInfoUtil.OS_IMEI);
		if(!TextUtils.isEmpty(imei))
			param += AppInfoUtil.OS_IMEI + "=" +imei;
		String appid = getAppid();
		if(!TextUtils.isEmpty(appid))
		{
			if(!TextUtils.isEmpty(param))
				param += "&";
			param += SpeechConstant.APPID + "=" + appid;
		}
		return param;
	}
	
	/**
	 * 获取当前应用的appid
	 * @return
	 */
	public static String getAppid()
	{	
		// 获取当前已经创建的对象
		return SpeechUtility.getUtility().getParameter(SpeechConstant.APPID);
	}
	
	/**
	 * 获取当前应用的MD5值
	 * @return
	 */
	public static String getMD5()
	{
		int code = 0;
        try {
        	if( MSC.isLoaded())
        	{
        		MSCSessionInfo info = new MSCSessionInfo();
        		code = MSC.QMSPGetParam(MscKeys.MD5.getBytes(), info);
        		if (code == 0) 
        		{
        			return new String(info.buffer).trim();
        		}
        	}
        } catch (UnsatisfiedLinkError e) {
        } catch (Exception e) {
		}
        return null;
	}
	
	/**
	 * 检测是否可能需要长时间语音输入，设置不同的输入限制、前后端点时间
	 * @param domain domain参数
	 * @return
	 */
	public static boolean isLongInput(String domain) {
		return null!=domain && (domain.contains(DOMAIN_SMS) || domain.contains(DOMAIN_IAT));
	}
	
	/**
	 * 语速-倍速适配，本地语速从0-100 扩充到 0-200
	 * @param params 参数
	 * @return
	 */
	private static void speedIncreaseAdaptive(HashParam params){
		//外部未传倍速参数
		final SpeechUtility utility = SpeechUtility.getUtility();
		if( null == utility ){
			return;
		}
		String verTts = utility.getParameter(MscKeys.VER_TTS);
		
		if (!params.hasKey(MscKeys.KEY_SPEED_INCREASE)) {
			int speed = params.getInt(SpeechConstant.SPEED, 50);
			if (speed <= 100) {
				params.putParam(SpeechConstant.SPEED, "" + speed);
				params.putParam(MscKeys.KEY_SPEED_INCREASE, "1");
			} else if ((100 < speed && speed <= 150)
					&&(TextUtils.isEmpty(verTts) || verTts.contains("5.5."))) {
				// 设置两倍速
				params.putParam(SpeechConstant.SPEED, "" + (speed - 50));
				params.putParam(MscKeys.KEY_SPEED_INCREASE, "2");
			} else if (100 < speed && speed <= 200) {
				// 设置两倍速
				params.putParam(SpeechConstant.SPEED, "" + (speed - 100));
				params.putParam(MscKeys.KEY_SPEED_INCREASE, "4");
			}
		}
	}

	public static int getDefaultBos( final MscHandler mscer ){
		return null!=mscer&&mscer.isLongInput() ? DEF_BOS_LONG : DEF_BOS_SHORT;
	}
	
	public static int getDefaultEos( final MscHandler mscer ){
		return null!=mscer&&mscer.isLongInput() ? DEF_EOS_LONG : DEF_EOS_SHORT;
	}
}
