package com.iflytek.cloud.util;


import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;

import com.iflytek.msc.MSC;

import java.security.SecureRandom;

/**
 * <h3>身份认证辅助类</h3>
 * <p>
 * 		身份认证时，辅助生成声纹随机密码，或转换人脸认证图。
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * 
 * @see com.iflytek.cloud.FaceDetector
 * @see com.iflytek.cloud.IdentityVerifier
 */
public class VerifierUtil {

	// 生成声纹验证数字密码用到的字典
	private static final String numPwdDict = "023456789";
	
	/**
	 * <h4>生成随机密码</h4>
	 * <p>
	 * 		当使用随机密码方式验证时，需要先通过本函数，生成一个指定长度的随机密码，
	 * 用于验证时由用户读这个随机密码，服务器通过音频里面随机码及声纹特征是否一致，
	 * 验证是否是同一个用户。
	 * </p>
	 * @param length
	 *            随机字符串长度，当前只支持8位。
	 * @return 随机字符串
	 * 
	 * @see com.iflytek.cloud.IdentityVerifier#writeData
	 */
	public static String generateNumberPassword(int length) {
		StringBuffer sb = new StringBuffer();
		SecureRandom random = new SecureRandom();
		// 生成第一个数字
		String selected = numPwdDict.charAt(random.nextInt(numPwdDict.length())) + "";
		sb.append(selected);

		// 生成余下的数字
		for (int i = 0; i < length - 1; i++) {
			Boolean isOkBoolean = false;
			
			while (!isOkBoolean) {
				selected = numPwdDict.charAt(random.nextInt(numPwdDict.length())) + "";
				// 该数字不与之前的重复
				if (sb.indexOf(selected) >= 0) {
					isOkBoolean = false;
					continue;
				}
				
				// 该数字与前一个数字乘积!=10
				if ((Integer.parseInt(sb.charAt(sb.length() - 1) + "") * Integer
						.parseInt(selected)) == 10) {
					isOkBoolean = false;
					continue;
				}
				
				isOkBoolean = true;
			}
			
			sb.append(selected);
		}
		
		return sb.toString();
	}

	/**
	 * <h4>ARGB彩图转灰度图</h4>
	 * <p>
	 * 		把输入的ARGB彩色图，转换并返回灰度图。
	 * </p>
	 * @param src ARGB彩图
	 * @return 灰度图
	 * 
	 * @see com.iflytek.cloud.IdentityVerifier#writeData
	 * @see com.iflytek.cloud.FaceDetector#detectGray(Bitmap)
	 */
	static public Bitmap ARGB2Gray(Bitmap src) {
		if(src.getConfig() == Config.ARGB_8888) {
			// TODO to gray
			Bitmap grayBmp = Bitmap.createBitmap(src.getWidth(), src.getHeight(),
					Bitmap.Config.ALPHA_8);
			if(!doARGB2Gray(src, grayBmp))
				return null;
			return grayBmp;
		} else
			return null;   
	}
	
	private static boolean doARGB2Gray(Bitmap src, Bitmap dst) {
		return MSC.doARGB2Gray(src, dst);
	}
	
	// 暂不支持nv21转bitmap
//	/**
//	 * 将Android的特殊格式快速转化成灰度图
//	 * 
//	 * @param nv21
//	 * Android读图特殊格式，可快速转化成灰度图，也可以直接作为灰度图输入
//	 * 
//	 * @param dst
//	 * 取出NV21的前部分（灰度图部分）返回
//	 */
//	public static void NV21ToGrayBitmap(byte[] nv21, Bitmap dst) {
//		FaceSDK.NV21ToGrayBitmap(nv21, dst);
//	}
//	
//	/**
//	 * 旋转像素长度1bit的图
//	 * 
//	 * @param src
//	 * 像素长度1bit的输入图
//	 * 
//	 * @param dst
//	 * 旋转后的图
//	 */
//	public static void RotateBitmapAlpha8(Bitmap src, Bitmap dst) {
//		FaceSDK.RotateBitmapAlpha8(src, dst);
//	}

	/**
	 * <h4>获取bitmap大小</h4>
	 * <p>
	 * 		获取bitmap所占的字节数
	 * </p>
	 * 
	 * @param bitmap 图形数据
	 * @return 所占的字节数
	 */
	static public int getBitmapsize(Bitmap bitmap) {
		return bitmap.getRowBytes() * bitmap.getHeight();
	}

	
}
