
package com.iflytek.cloud.util.contacts.impl;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.provider.Contacts;
import android.provider.Contacts.People;

import java.util.ArrayList;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.util.contacts.ContactManagerImpl;
import com.iflytek.cloud.util.contacts.PhoneNumberUtil;
import com.iflytek.cloud.util.contacts.interfaces.ContactAccessor;

@SuppressWarnings("deprecation")
public class ContactAccessorSdk3_4 extends ContactAccessor {
	
    private static final String[] CONTACT_PROJECTION_16 = new String[] {
        Contacts.People._ID, Contacts.People.NAME};
    
    private static final String[] CONTACT_INFO_PROJECTION_16 = new String[] {
        Contacts.People.NAME, Contacts.People.NUMBER, Contacts.People._ID};
    
    private static final String[] ID_PROJECTION_16 = new String[] {
        Contacts.Phones.PERSON_ID};
    
    private static final String[] LOOKUP_PROJECTION_16 = new String[] {
        Contacts.Phones.DISPLAY_NAME};
    
    private static final String[] PHONE_PROJECTION_16 = new String[] {
        Contacts.Phones.NUMBER, Contacts.Phones.TYPE, Contacts.Phones.NAME};
    
    private static final String[] CONTACT_ITEM_PROJECTION_16 = new String[] {
        Contacts.People._ID, Contacts.People.NAME, Contacts.Phones.NUMBER,
        Contacts.People.TYPE};
    
    private static final String[] CONTACT_HAS_NUMBER_PROJECTION_16 = new String[] {
        Contacts.Phones.NUMBER
    };
    
    public ContactAccessorSdk3_4(Context context) {
        super(context);
        setNumberType(context);
    } 

    @Override
    public Uri getContactUri() {
        return Contacts.People.CONTENT_URI;
    }

    @Override
    public String queryContactIdByNumber(String number) {
        Cursor cursor = null;
        String contactId = null;
        
        try {
            if (null == number) {
                DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_16----------id is null");
            } else {
                cursor = mContext.getContentResolver().query(
                        Contacts.Phones.CONTENT_URI, ID_PROJECTION_16,
                        "phones.number = " + number, null, null);

                if (null == cursor) {
                	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_16---------cursor is null");
                } else if (0 == cursor.getCount()) {
                	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_16---------cursor count is 0");
                } else {
                    while (cursor.moveToNext()) {
                        contactId = cursor.getString(0);
                        break;
                    }
                }
            }
            return contactId;
        } catch (Exception e) {
            DebugLog.LogE(e);
            return contactId;
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
    }

    @Override
    public ArrayList<String> queryPhoneNumberByName(String name) {
        Cursor cursor = null;
        String phoneNumber = null;
        String nameWhere = Contacts.People.NAME + " = " + name;
        ArrayList<String> tmpList = new ArrayList<String>();
        
        try {
            cursor = mContext.getContentResolver().query(
                    Contacts.People.CONTENT_URI, CONTACT_ITEM_PROJECTION_16, nameWhere,
                    null, Contacts.People.NAME);

            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_16-------cursor is null");
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_16-------cursor getCount == 0");
            } else {
                while (cursor.moveToNext()) {
                    phoneNumber = cursor.getString(cursor
                            .getColumnIndex(Contacts.People.NUMBER));
                    tmpList.add(phoneNumber);
                }
                
                DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_16-------cursor getCount = "
                        + cursor.getCount());
            }
            
            return tmpList;
        } catch (Exception e) {
            DebugLog.LogE(e);
            return null;
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
    
    }
    @Override
    public void insertIntentContact(String name, String number) {
        try {
            Uri insertUri = People.CONTENT_URI;
            Intent intent = new Intent(Intent.ACTION_INSERT, insertUri);
            if (null != name) {
                intent.putExtra(Contacts.Intents.Insert.NAME, name);
            }
            if (null != number) {
                intent.putExtra(Contacts.Intents.Insert.PHONE, number);
            }
            intent.putExtra(Contacts.Intents.Insert.PHONE_TYPE,
                  Contacts.Phones.TYPE_MOBILE);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK
                  | Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
            mContext.startActivity(intent);
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
    }

    @Override
    public boolean hasContactNumber(String name) {
        Cursor cursor = null;
        String nameWhere = Contacts.People.NAME + " = ?";
        String[] whereParams = new String[] {name}; 
        boolean hasNumberFlag = false;
        
        try {
            cursor = mContext.getContentResolver().query(
                    Contacts.People.CONTENT_URI, CONTACT_HAS_NUMBER_PROJECTION_16, nameWhere,
                    whereParams, Contacts.People.NAME);
            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "has_contact_number_16 ----------------------- cursor is null");
                return hasNumberFlag;
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "has_contact_number_16 ----------------------- cursor size is 0");
                return hasNumberFlag;
            } else {
                while (cursor.moveToNext()) {
                    String number = cursor.getString(cursor.getColumnIndex(Contacts.People.NUMBER));
                    if (null != number) {
                        hasNumberFlag = true; 
                    } else {
                        hasNumberFlag = false;
                    }
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
        return hasNumberFlag;
    }

    @Override
    protected String[] getContactProjection() {
        return CONTACT_PROJECTION_16;
    }
    
    protected String[] getContactInfoProjection() {
        return CONTACT_INFO_PROJECTION_16;
    }
   
    @Override
    protected String[] getLookUpProjection() {
        return LOOKUP_PROJECTION_16;
    }

    @Override
    protected String getOrderByKey() {
        return Contacts.People.NAME;
    }

    @Override
    protected Uri getLookUpUri() {
        return Contacts.People.CONTENT_URI;
    }

    @Override
    protected Uri getContactFilterUri() {
        return Contacts.Phones.CONTENT_FILTER_URL;
    }

    @Override
    public Uri getPhoneUri() {
        return Contacts.Phones.CONTENT_URI;
    }

    @Override
    public String[] getPhoneProjection() {
        return PHONE_PROJECTION_16;
    }

    @Override
    public String getContactSetWhereStr(boolean fuzzyFlag) {
        if (fuzzyFlag) {
            return Contacts.Phones.NAME + " like ?";
        }
        return Contacts.Phones.NAME + " = ?";
    }

    @Override
    public String[] getContactSetWhereParams(String name, boolean fuzzyFlag) {
        if (fuzzyFlag) {
            return new String[] {"%" + PhoneNumberUtil.filterSqlName(name) + "%"};
        }
        return new String[] {name};
    }

	@Override
	public byte[] queryContactPhotoDataByName(String name) {
		return null;
	}

	@Override
	public Uri insertNewContact(String name, String number) {
		try {
			ContentValues values = new ContentValues();
			// 插入联系人姓名
			values.put(Contacts.People.NAME, name);  

/* STARRED 0 = Contacts, 1 = Favorites */

			values.put(Contacts.People.STARRED, 0);
			Uri newPersonUri = Contacts.People.createPersonInMyContactsGroup(  
				    mContext.getContentResolver(), values);
			
			// 插入联系人号码
			values.clear();
			Uri uri = Uri.withAppendedPath(newPersonUri, Contacts.People.Phones.CONTENT_DIRECTORY);  
			values.put(Contacts.Phones.TYPE, Contacts.Phones.TYPE_HOME);  
			values.put(Contacts.Phones.NUMBER, number);  
			    mContext.getContentResolver().insert(uri, values);  
			return mContext.getContentResolver().insert(uri, values);
		} catch (Exception e) {
			DebugLog.LogE(e);
			return null;
		}
		
	}

}
