
package com.iflytek.cloud.util.contacts.interfaces;

import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.provider.CallLog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.iflytek.cloud.msc.util.Encrypter;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.util.contacts.ContactManagerImpl;
import com.iflytek.cloud.util.contacts.PhoneNumberUtil;
import com.iflytek.cloud.util.contacts.entities.ContactItem;
import com.iflytek.cloud.util.contacts.entities.ContactSet;

/**
 * 联系人查询基类
 * @author weiwei
 *
 */

public abstract class ContactAccessor {
	
    protected static final String SIM_ID = "_id";
    protected static final String SIM_NAME = "name";
    protected static final String SIM_NUMBER = "number";
    protected static final String SIM_URI = "content://icc/adn";
    protected static final int CALL_LOG_SIZE = 10;
    
    protected static final String[] CALL_LOG_PROJECTION = new String[] {
        CallLog.Calls.NUMBER, CallLog.Calls.CACHED_NAME, CallLog.Calls.DATE};
    
    protected Context mContext = null;
    
    // 联系人号码类别
    protected static final int TYPE_HOME = 1;
    protected static final int TYPE_MOBILE = 2;
    protected static final int TYPE_WORK = 3;
    protected static final int TYPE_WORK_FAX = 4;
    protected static final int TYPE_HOME_FAX = 5;
    protected static final int TYPE_CALL = 6;
    protected static final int TYPE_OTHER = 7;
    protected static final int TYPE_SIM = 9;
    protected static final int TYPE_CUSTOM = 8;
    protected static final int TYPE_COUNT = 100;
    protected static String[] mTypes;
    
    public ContactAccessor(Context context) {
        mContext = context;
    }
    

/**
     * 获取联系人查询信息，包括联系人id，联系人姓名
     * @return 查询信息数组
     */

    protected abstract String[] getContactProjection();
    

/**
     * 获取联系人查询信息，包括联系人id，联系人姓名，号码
     * @return 查询信息数组
     */

    protected abstract String[] getContactInfoProjection();
    

/**
     * 获取联系人快捷查询信息，包括联系人的姓名
     * @return 
     */

    protected abstract String[] getLookUpProjection();
    

/**
     * 获取联系人查询排序方式
     * @return 查询排序方式
     */

    protected abstract String getOrderByKey();
    

/**
     * 获取lookup查询的URI
     * @return
     */

    protected abstract Uri getLookUpUri();
    

/**
     * 获取Contact Filter查询URI
     * @return
     */

    protected abstract Uri getContactFilterUri();
    

/**
     * 获取联系人电话信息URI
     * @return
     */

    public abstract Uri getPhoneUri();
    

/**
     * 获取联系人号码相关信息，包括联系人姓名，号码，号码类别
     * @return
     */

    public abstract String[] getPhoneProjection();
    

/**
     * 查询联系人集合条件语句
     * @param fuzzyFlag 是否模糊匹配
     * @return
     */

    public abstract String getContactSetWhereStr(boolean fuzzyFlag);
    

/**
     * 查询联系人集合条件参数数组
     * @param fuzzyFlag 是否模糊匹配
     * @return
     */

    public abstract String[] getContactSetWhereParams(String name, boolean fuzzyFlag);
    

/**
     * 查询手机内所有联系人姓名
     * @return 联系人ID/姓名映射列表
     */

    public HashMap<String, String> queryContactNames() {
        Cursor cursor = null;
        String contactName = null;
        String contactId = null;
        String[] contactProjection = getContactProjection();
        HashMap<String, String> contactNameHashMap = new HashMap<String, String>();
        try {
            cursor = mContext.getContentResolver().query(getContactUri(), contactProjection, null, null, getOrderByKey());
            if (null == cursor) {
           // 	DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts------cursor is null");
            } else if (0 == cursor.getCount()) {
          //  	DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts------cursor getCount == 0");
            } else {
                while (cursor.moveToNext()) {
                    contactName = cursor.getString(cursor.getColumnIndex(contactProjection[0]));
                    contactId = cursor.getString(cursor.getColumnIndex(contactProjection[1])); 
                    if (contactName != null) {
                        contactNameHashMap.put(contactId, contactName);
                    }
                }
                DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts_20------count = "  + cursor.getCount());
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
        return contactNameHashMap;
    }
    
    public HashMap<String, String> queryContactInfo() {
    	Cursor cursor = null;
    	String contactName = null;
        String contactId = null;
        String contactNumber = null;
        String contactInfo = null;
        String[] contactProjection = getContactInfoProjection(); 
        
        HashMap<String, String> contactInfoHashMap = new HashMap<String, String>();
        try {
        	cursor = mContext.getContentResolver().query(getPhoneUri(), contactProjection
            		, null, null, null);
            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts------cursor is null");
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts------cursor getCount == 0");
            } else {
                while (cursor.moveToNext()) {
                	contactName = cursor.getString(0);
                	contactNumber = cursor.getString(1);
                	contactNumber = PhoneNumberUtil.filterHeadNumber(contactNumber);
                	contactNumber = Encrypter.MD5(contactNumber);

                	contactId = cursor.getString(2);
                	
                	if(contactInfoHashMap != null && contactInfoHashMap.containsKey(contactId))
                	{
                		contactInfo = contactInfoHashMap.get(contactId) + "#" + contactNumber;
                		contactInfoHashMap.remove(contactId);
                		contactInfoHashMap.put(contactId, contactInfo);
                	}else
                		contactInfoHashMap.put(contactId, 
                				contactName + "#" + contactNumber);
                }
                DebugLog.LogD(ContactManagerImpl.TAG, "queryContacts_20------count = "  + cursor.getCount());
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
        return contactInfoHashMap;
    }
    

/**
     * 查询SIM卡联系人信息
     * @return SIM卡联系人信息ContactItem
     */

    public List<ContactItem> querySIMContactInfo() {
        Cursor simCursor = null;
        String simName = null;
        String simId = null;
        String simNum = null;
        List<ContactItem> simContactItems = new ArrayList<ContactItem>();
        
        try {
            simCursor = mContext.getContentResolver().query(Uri.parse(SIM_URI), null, null, null,
                    null);
            if (null != simCursor && simCursor.getCount() > 0) {
                while (simCursor.moveToNext()) {
                    simName = simCursor.getString(simCursor.getColumnIndex(SIM_NAME));
                    simId = simCursor.getString(simCursor.getColumnIndex(SIM_ID));
                    simNum = simCursor.getString(simCursor.getColumnIndex(SIM_NUMBER));
                    simNum = PhoneNumberUtil.filterHeadNumber(simNum);
                    simNum = Encrypter.MD5(simNum);
                    
                    if (simName != null) {
                        ContactItem contactItem = new ContactItem(simId, simName, null, simNum,
                                null, mTypes[TYPE_SIM]);
                        simContactItems.add(contactItem);
                    }
                }
                DebugLog.LogD(ContactManagerImpl.TAG, "querySIM-------count = " + simCursor.getCount());
            } else {
            	DebugLog.LogD(ContactManagerImpl.TAG, "querySIM-------cursor getCount = 0 or cursor is null");
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != simCursor) {
                simCursor.close();
            }
        }
        return simContactItems;
    }
    

/**
     * 查询通话记录uri
     * @return
     */

    public Uri getCallLogUri() {
    	return CallLog.Calls.CONTENT_URI;
    }
    

/**
     * 获取联系人数据URI
     * @return
     */

    public abstract Uri getContactUri();
    

/**
     * 根据号码查询联系人ID
     * @param number 联系人号码
     * @return ID 联系人ID
     */

    public abstract String queryContactIdByNumber(String number);
    

/**
     * 根据号码查询手机联系人名
     * @param number 联系人号码
     * @return Name 联系人姓名
     */

    public String queryContactNameByNumber(String number) {
        Uri uri = Uri.withAppendedPath(getContactFilterUri(), Uri.encode(number)); 
        Cursor cursor = null;
        String name = null;
        String[] lookUpProjection = getLookUpProjection();

        try {
            cursor = mContext.getContentResolver().query(
                    uri, lookUpProjection, null, null, null);
            
            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactByNumer--------cursor is null");
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactByNumer--------cursor count is 0");
            } else if (cursor.moveToFirst()) {
                name = cursor.getString(0);
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
        
        return name;
    }
    

/**
     * 根据号码查询SIM卡联系人名
     * @param number 联系人号码
     * @return Name 联系人姓名
     */

    protected String querySIMNameByNumber(String number) {
        Cursor simCursor = null;
        String simName = null;
        String simNumber = null;
        String[] projection = new String[] {SIM_NAME};
        
        try {
            simCursor = mContext.getContentResolver().query(
                    Uri.parse(SIM_URI), projection, null, null, null);
            if (null == simCursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "querySIMByNumer----cursor is null");
            } else if (0 == simCursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "querySIMByNumer----cursor count is 0");
            } else {
                while (simCursor.moveToNext()) {
                    simNumber = simCursor.getString(simCursor.getColumnIndex(SIM_NUMBER));
                    if (null != simNumber && simNumber.equals(number)) {
                        simName = simCursor.getString(simCursor.getColumnIndex(SIM_NAME));
                        break;
                    }
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != simCursor) {
                simCursor.close();
            }
        }
        
        return simName;
    }
    

/**
     * 根据姓名查询电话号码
     * @param name 联系人姓名
     * @return 电话号码list
     */

    public abstract ArrayList<String> queryPhoneNumberByName(String name);
    
    

/**
     *  在通讯录中插入新的联系人
     * @param number
     */

    public abstract void insertIntentContact(String name, String number);


/**
     * 根据联系人姓名查询联系人相关信息
     * @param name 联系人的姓名
     * @param date 最近联系人查询的时间
     * @return ContactSet 联系人集合
     */

    public ContactSet queryContactSet(String date, String name) {
        Cursor cursor = null;
        String number = null;
        String numberType = null;
        int type = 0;
        ContactSet contactSet = null;
        ArrayList<String> tmpNumberList = new ArrayList<String>();
        HashMap<String, String> tmpNumberTypeHashMap = new HashMap<String, String>();
        String[] projection = getPhoneProjection();
        
        try {
            cursor = mContext.getContentResolver().query(getPhoneUri(), projection, getContactSetWhereStr(false), 
                    getContactSetWhereParams(name, false), null);
            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactSet ----------------cursor is null");
                return contactSet;
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactSet ----------------cursor size is 0");
                return contactSet;
            } else {
                while (cursor.moveToNext()) {
                    number = PhoneNumberUtil.filterEmptyNumber(cursor.getString(cursor.getColumnIndex(projection[0])));
//                	number = cursor.getString(cursor.getColumnIndex(projection[0]));
                    type = cursor.getInt(cursor.getColumnIndex(projection[1]));
                    
                    numberType = mTypes[type];
                    if (!number.equals("") && !tmpNumberTypeHashMap.containsKey(number)) {
                        tmpNumberTypeHashMap.put(number, numberType);
                        tmpNumberList.add(number);
                    }
                }
                contactSet = new ContactSet(name, date);
                contactSet.setNumbers(tmpNumberList);
                contactSet.setNumberTypeHashMap(tmpNumberTypeHashMap);
                return contactSet;
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
            return null;
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
    }
    

/**
     * 模糊查询联系人信息
     * @param date 最近联系人查询时间
     * @param name 联系人姓名
     * @return 联系人集合list
     */

    public ArrayList<ContactSet> fuzzyQueryContactSets(String date, String name) {
        Cursor cursor = null;
        String number = null;
        String numberType = null;
        String disPlayName = null;
        int type = 0;
        HashMap<String, ContactSet> tmpContactHashMap = new HashMap<String, ContactSet>();
        ArrayList<ContactSet> contactSets = new ArrayList<ContactSet>();
        String[] projection = getPhoneProjection();
        
        try {
            cursor = mContext.getContentResolver().query(getPhoneUri(),
                    projection, getContactSetWhereStr(true), getContactSetWhereParams(name, true), null);
            if (null == cursor) {
         //   	DebugLog.LogD(ContactManagerImpl.TAG, "fuzzy_queryContactSet ----------------cursor is null");
                return contactSets;
            } else if (0 == cursor.getCount()) {
        //    	DebugLog.LogD(ContactManagerImpl.TAG, "fuzzy_queryContactSet ----------------cursor size is 0");
                return contactSets;
            } else {
                while (cursor.moveToNext()) {
                    number = PhoneNumberUtil.filterEmptyNumber(cursor.getString(cursor.getColumnIndex(projection[0])));
//                	number = cursor.getString(cursor.getColumnIndex(projection[0]));
                    type = cursor.getInt(cursor.getColumnIndex(projection[1]));
                    disPlayName = cursor.getString(cursor.getColumnIndex(projection[2]));
                    numberType = mTypes[type];
                    if (!number.equals("") && !tmpContactHashMap.containsKey(disPlayName)) {
                        ContactSet set = new ContactSet(disPlayName, date);
                        ArrayList<String> tmpNumberList = new ArrayList<String>();
                        HashMap<String, String> tmpTypeHashMap = new HashMap<String, String>();
                        tmpNumberList.add(number);
                        tmpTypeHashMap.put(number, numberType);
                        set.setNumbers(tmpNumberList);
                        set.setNumberTypeHashMap(tmpTypeHashMap);
                        tmpContactHashMap.put(disPlayName, set);
                    } else {
                        ContactSet set = tmpContactHashMap.get(disPlayName);
                        if (!number.equals("") && !set.getNumbers().contains(number)) {
                            set.getNumbers().add(number);
                            set.getNumberTypesHashMap().put(number, numberType);
                        }
                    }
                }
                // 首先将aitalk返回的名称保存到列表首位
                ContactSet set = tmpContactHashMap.get(name);
                if (null != set) {
                	contactSets.add(set);
                	tmpContactHashMap.remove(name);
                }
				Iterator<String> iterator = tmpContactHashMap.keySet().iterator();
				while (iterator.hasNext()) {
					contactSets.add(tmpContactHashMap.get(iterator.next()));
				}
                
//				DebugLog.LogD(ContactManagerImpl.TAG, "fuzzy_queryContactSet ----------------cursor size = " + cursor.getCount());
                return contactSets;
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
            return contactSets;
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
    }
    

/**
     * 根据姓名查询联系人头像
     * @param name
     * @return
     */

    public abstract byte[] queryContactPhotoDataByName(String name);
    

/**
     * 根据姓名和号码新建联系人
     * @param name
     * @param number
     */

    public abstract Uri insertNewContact(String name, String number);
    

/**
     * 根据名称查询联系人号码
     * @param name
     * @return true 联系人有号码 false 联系人没有号码
     */

    public abstract boolean hasContactNumber(String name);
    	
    private static final String PHONE_TYPE_HOME = "住宅";
    private static final String PHONE_TYPE_MOBILE = "手机";
    private static final String PHONE_TYPE_WORK_FAX = "工作传真";
    private static final String PHONE_TYPE_HOME_FAX = "住宅传真";
    private static final String PHONE_TYPE_CALL = "寻呼机";
    private static final String PHONE_TYPE_CUSTOM = "自定义电话";
    private static final String PHONE_TYPE_WORK = "工作";
    private static final String PHONE_TYPE_OTHER = "其他";
    private static final String PHONE_TYPE_SIM = "SIM卡";
    
    protected void setNumberType(Context context) {
        mTypes = new String[TYPE_COUNT];
        mTypes[0] = PHONE_TYPE_OTHER;
        mTypes[TYPE_HOME] = PHONE_TYPE_HOME;
        mTypes[TYPE_MOBILE] = PHONE_TYPE_MOBILE;
        mTypes[TYPE_WORK] = PHONE_TYPE_WORK;
        mTypes[TYPE_WORK_FAX] = PHONE_TYPE_WORK_FAX;
        mTypes[TYPE_HOME_FAX] = PHONE_TYPE_HOME_FAX;
        mTypes[TYPE_CALL] = PHONE_TYPE_CALL;
        mTypes[TYPE_OTHER] = PHONE_TYPE_OTHER;
        mTypes[TYPE_SIM] = PHONE_TYPE_SIM;
        for (int i = TYPE_SIM + 1; i < mTypes.length; i++) {
            mTypes[i] = PHONE_TYPE_CUSTOM;
        }
    }
    
}
