package com.iflytek.cloud.speech.impl;

import org.json.JSONObject;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RequestListener;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.HttpRequest;
import com.iflytek.cloud.msc.util.HttpRequest.HttpRequestListener;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;


public class FaceRequestImpl extends SpeechInterface {

	//注意：如果缺少域名最后的斜杠会导致2.3系统无法访问
//	private String LINK_URL ="http://wfr.openspeech.cn/";
	private String LINK_URL ="http://openapi.openspeech.cn/webapi/wfr.do";

	//版本协议号
	private String LINK_PARAM ="pver=1.0";
	private Context  mContext = null;
	private HttpRequest mHttpRequest = null;
	private RequestListener mListener;

	public FaceRequestImpl(Context context,HashParam param) {
		mSessionParams = param;
		mContext = context; 
		mHttpRequest = new HttpRequest();
	}

	public int sendRequest(byte[] data,RequestListener listener) {
		try {
			mListener = listener;
			SpeechUtility utility = SpeechUtility.getUtility();
			if (utility == null) {
				return ErrorCode.MSP_ERROR_NOT_INIT;
			}
			String url = mSessionParams.remove( MscKeys.SERVER_URL );
			if( TextUtils.isEmpty(url) ){
				url = LINK_URL;
			}
			
			//组装参数
			String paramString = ParamBuilder.composeWfrSessionParam(mContext, mSessionParams);
			
			//put 超时时间
			mHttpRequest.setTimeOut(mSessionParams.getInt(SpeechConstant.NET_TIMEOUT, 20000));
			
			mHttpRequest.setConectType(HttpRequest.CONNECT_POST);
			mHttpRequest.setRequest(url, LINK_PARAM, data,paramString);
			mHttpRequest.startRequest(mHttpRequestListener);
//			// 先清空日志
//			PerfLogger.clear();
			// 记录发送请求的时间点
			PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
			return ErrorCode.SUCCESS;
		}
		catch (Exception e) {
			return ErrorCode.ERROR_UNKNOWN;
		}
	}

	public void cancel() {
		mHttpRequest.cancel();
		mHttpRequest = null;
	}

	public boolean destroy(){
		return super.destroy();
	}
	
	HttpRequestListener mHttpRequestListener = new HttpRequestListener() {

		@Override
		public void onError(SpeechError error) {
			if(error != null)
			{ 	
				DebugLog.LogE("upload error. please check net state:"+error.getErrorCode());
			} else {
				DebugLog.LogD("upload succeed");
			}
			if (mListener != null) {
				mListener.onCompleted(error);
			}
		}

		@Override
		public void onResult(HttpRequest request, byte[] data) {
			// 记录从发送请求到获取结果的时长
			try {				
				// 有效数据
				if(data != null)
				{
					String result = new String(data,DataUtil.UTF8);
					//TODO 加解密
					DebugLog.LogD(result);
					// 检测是否发生错误
					JSONObject object = new JSONObject(result);
					int  ret = Integer.parseInt(object.getString("ret"));
					// 记录整个返回的json结果
					if(ErrorCode.SUCCESS != ret)
					{
						onError(new SpeechError(ret,SpeechConstant.ENG_WFR));
					}else {
						if (mListener != null) {
							PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
							mListener.onBufferReceived(data);
						}
						onError(null);
					}
				}					
			} catch (Exception e) {
				onError(new SpeechError(ErrorCode.ERROR_INVALID_RESULT));
			}
		}	
	};
}
